unit FMandelTest;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, Math, ExtCtrls, ShellAPI;

type
  TMainForm = class(TForm)
    Image1: TImage;
    Shape1: TShape;
    Label1: TLabel;
    cbThreads: TComboBox;
    lbCoreCount: TLabel;
    Reset: TButton;
    Label2: TLabel;
    Label3: TLabel;
    Button1: TButton;
    Button2: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Image1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Image1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure cbThreadsChange(Sender: TObject);
    procedure Label3Click(Sender: TObject);
    procedure ResetClick(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Dclarations prives }
    procedure NextPixel(Sender: TObject; Index: Integer);
  public
    { Dclarations publiques }
    FBitmap : TBitmap;
    mbX, mbY : Integer;

    procedure ComputeMandel;
  end;

var
  MainForm: TMainForm;

implementation

uses Mandelbrot, Parallelization;

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
var
   i : Integer;
begin
   FBitmap:=TBitmap.Create;
   FBitmap.PixelFormat:=pf32bit;
   //FBitmap.SetSize(Image1.Width, Image1.Height);
   FBitmap.Width := Image1.Width;
   FBitmap.Height := Image1.Height;

   Image1.Picture.Bitmap:=FBitmap;

   lbCoreCount.Caption := IntToStr(MandelBrot.cores) + ' cores on this computer';

   ComputeMandel;
end;

procedure TMainForm.Image1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
   mbX:=X;
   mbY:=Y;
end;

procedure TMainForm.Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
   s : Integer;
begin
   if ssLeft in Shift then begin
      s := Max(X-mbX, Y-mbY);
      if s>0 then begin
         Shape1.SetBounds(mbX+Image1.Left, mbY+Image1.Top, s, s);
         Shape1.Visible:=True;
      end;
   end;
end;

procedure TMainForm.Image1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
   s : Integer;
   pw, qw : Double;
begin
   Shape1.Visible:=False;

   s:=Max(X-mbX, Y-mbY);
   if (s>3) then begin
      X := mbX + s;
      Y := mbY + s;
      pw := pmax - pmin;
      pmin := pmin + mbX * pw / FBitmap.Width;
      pmax := pmax - (FBitmap.Width - X) * pw / FBitmap.Width;
      qw := qmax - qmin;
      qmin := qmin + (FBitmap.Height - Y) * qw / FBitmap.Height;
      qmax := qmax - mbY * qw / FBitmap.Height;

      ComputeMandel;
   end;
end;

// ComputeMandel
//
procedure TMainForm.ComputeMandel;
var
   start, stop, freq : Int64;
   Str  : string;
   Index: Integer;
begin

   QueryPerformanceCounter(start);

   MandelBrot.Compute(cbThreads.ItemIndex);

   QueryPerformanceCounter(stop);
   QueryPerformanceFrequency(freq);

   SetDIBitsToDevice(
    FBitmap.Canvas.Handle,
    0, 0, IMAGE_SIZE, IMAGE_SIZE,
    0, 0, 0, IMAGE_SIZE,
    @MandelBrot.pixels,
    MandelBrot.DIBInfo,
    0
  );
   Image1.Picture.Bitmap:=FBitmap;

   Str := Format('%.1f milliseconds', [(stop-start)/freq*1000]);
   for Index := 0 to Length(counters) - 1 do
   begin
     Str := Str + ', ' + IntToStr(counters[Index]);
   end;
   Caption := Str;
end;


procedure TMainForm.cbThreadsChange(Sender: TObject);
begin
  ComputeMandel;
end;

procedure TMainForm.Label3Click(Sender: TObject);
var
  Str: string;
begin
  Str := TLabel(Sender).Hint;
  ShellExecute(Handle, nil, PChar(str), nil, nil, SW_SHOW);
end;

procedure TMainForm.ResetClick(Sender: TObject);
begin
  MandelBrot.Reset;
  ComputeMandel;
end;

procedure TMainForm.Button2Click(Sender: TObject);
var
  start, stop, freq : Int64;
  P: TParallelizer;
begin
  QueryPerformanceCounter(start);

  P := TParallelizer.Create;
  try
    P.OnIteration := NextPixel;
    P.Iterate(480 * 480);
  finally
    P.Free;
  end;

  QueryPerformanceCounter(stop);
  QueryPerformanceFrequency(freq);

  Caption := Format('%.1f milliseconds', [(stop-start)/freq*1000]);

  SetDIBitsToDevice(
    FBitmap.Canvas.Handle,
    0, 0, IMAGE_SIZE, IMAGE_SIZE,
    0, 0, 0, IMAGE_SIZE,
    @MandelBrot.pixels,
    MandelBrot.DIBInfo,
    0
  );
  Image1.Picture.Bitmap:=FBitmap;
end;

procedure TMainForm.NextPixel(Sender: TObject; Index: Integer);
begin
  Mandelbrot.ComputePixel(Index mod 480, Index div 480);
end;

end.
